@@ -21,6 +21,7 @@ from fluxloop.buffer import EventBuffer
 from fluxloop.schemas import ExperimentConfig, PersonaConfig
 from rich.console import Console
 
+from .environment import load_env_chain
 from .target_loader import TargetLoader
 from .arg_binder import ArgBinder
 
@@ -97,22 +98,17 @@ class ExperimentRunner:
         """Load environment variables from .env and runner settings."""
 
         source_dir = self.config.get_source_dir()
-        env_candidates: List[Path] = []
 
-        if source_dir:
-            env_candidates.append(source_dir / ".env")
-            parent = source_dir.parent
-            if parent != source_dir:
-                env_candidates.append(parent / ".env")
+        def _log_env_error(path: Path, exc: Exception) -> None:
+            console.log(
+                f"[yellow]Warning:[/yellow] Failed to load environment from {path}: {exc}"
+            )
 
-        for candidate in env_candidates:
-            if candidate.exists():
-                try:
-                    fluxloop.load_env(candidate, override=True, refresh_config=True)
-                except Exception:
-                    console.log(f"[yellow]Warning:[/yellow] Failed to load environment from {candidate}")
-                else:
-                    break
+        load_env_chain(
+            source_dir,
+            refresh_config=True,
+            on_error=_log_env_error,
+        )
 
         env_vars = getattr(self.config.runner, "environment_vars", {}) or {}
         for key, value in env_vars.items():
@@ -817,20 +813,20 @@ class ExperimentRunner:
             for source_path in existing:
                 try:
                     with source_path.open("r", encoding="utf-8") as src:
-            for line in src:
-                if not line.strip():
-                    continue
-                try:
-                    record = json.loads(line)
-                except json.JSONDecodeError:
-                    continue
-                if record.get("trace_id") in trace_ids:
+                        for line in src:
+                            if not line.strip():
+                                continue
+                            try:
+                                record = json.loads(line)
+                            except json.JSONDecodeError:
+                                continue
+                            if record.get("trace_id") in trace_ids:
                                 key = (record.get("id"), record.get("start_time"))
                                 if key in seen_lines:
                                     continue
-                    dst.write(json.dumps(record) + "\n")
+                                dst.write(json.dumps(record) + "\n")
                                 seen_lines.add(key)
-                    copied += 1
+                                copied += 1
                 except OSError:
                     continue
 
